package com.zanxiang.game.module.manage.rpc.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.zanxiang.game.module.base.pojo.dto.SendMsgDto;
import com.zanxiang.game.module.base.util.PageUtil;
import com.zanxiang.game.module.base.pojo.params.SendMsgTaskParam;
import com.zanxiang.game.module.base.pojo.vo.SendMsgVo;
import com.zanxiang.game.module.base.rpc.ICPSendMsgRpc;
import com.zanxiang.game.module.manage.enums.CpSendMsgTaskStatusEnum;
import com.zanxiang.game.module.manage.enums.CpSendMsgTaskTypeEnum;
import com.zanxiang.game.module.manage.pojo.vo.CpVO;
import com.zanxiang.game.module.manage.service.ICPSendMsgTaskService;
import com.zanxiang.game.module.mybatis.entity.CpSendMsgTask;
import com.zanxiang.module.util.bean.BeanUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.DubboService;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@DubboService
@Slf4j
public class CpSendMsgRpcImpl implements ICPSendMsgRpc {

    @Resource
    private ICPSendMsgTaskService sendMsgTaskService;

    @Override
    public ResultVO<Boolean> sendMsg(SendMsgDto sendMsgDto) {
        try {
            // 数据校验
            if (sendMsgDto == null || sendMsgDto.getMsg() == null || sendMsgDto.getGameId() == null || sendMsgDto.getCreateBy() == null || sendMsgDto.getRoles() == null || sendMsgDto.getRoles().isEmpty()) {
                return ResultVO.fail("参数不能为空");
            }
            //创建任务
            CpSendMsgTask cpSendMsgTask = createSendMsgTask(sendMsgDto);
            //保存任务
            sendMsgTaskService.save(cpSendMsgTask);
            //todo：发送消息
            log.info("发送消息成功");
            return ResultVO.ok();
        } catch (Exception e) {
            log.error("发送消息异常, sendMsgDto : {}, e : {}", sendMsgDto, e.getMessage());
            return ResultVO.fail("发送消息异常");
        }
    }

    @Override
    public PageUtil<SendMsgVo> getSendMsgTaskList(SendMsgTaskParam param) {
        Page<CpSendMsgTask> page = sendMsgTaskService.page(new Page<>(param.getPageNum(), param.getPageSize()),
                new LambdaQueryWrapper<CpSendMsgTask>()
                        .eq(param.getGameId() != null, CpSendMsgTask::getGameId, param.getGameId())
                        .orderByDesc(CpSendMsgTask::getCreateTime));
        List<CpSendMsgTask> cpSendMsgTasks = page.getRecords();
        List<SendMsgVo> sendMsgVos = toVo(cpSendMsgTasks);
        return new PageUtil<>(sendMsgVos, page.getTotal(), param.getPageSize(), param.getPageNum(),0);
    }

    private List<SendMsgVo> toVo(List<CpSendMsgTask> cpSendMsgTasks) {
        if (Objects.isNull(cpSendMsgTasks) || cpSendMsgTasks.isEmpty()) {
            return null;
        }
        return cpSendMsgTasks.stream()
                .map(task -> BeanUtil.copy(task, SendMsgVo.class))
                .collect(Collectors.toList());
    }


    private CpSendMsgTask createSendMsgTask(SendMsgDto dto) {
        CpSendMsgTask cpSendMsgTask = new CpSendMsgTask();
        cpSendMsgTask.setMsg(dto.getMsg());
        cpSendMsgTask.setRoleIdCount((long) dto.getRoles().size());
        cpSendMsgTask.setTaskName(dto.getTaskName());
        cpSendMsgTask.setTaskCondition(dto.getSendConditionJson());
        cpSendMsgTask.setGameId(dto.getGameId());
        //任务类型
        cpSendMsgTask.setType(CpSendMsgTaskTypeEnum.IMMEDIATE_TASK.getValue());
        //任务状态
        cpSendMsgTask.setStatus(CpSendMsgTaskStatusEnum.WAIT_SEND.getValue());
        cpSendMsgTask.setCreateBy(dto.getCreateBy());
        cpSendMsgTask.setCreateTime(LocalDateTime.now(ZoneId.of("UTC")));
        cpSendMsgTask.setUpdateTime(LocalDateTime.now(ZoneId.of("UTC")));
        return cpSendMsgTask;
    }
}
